use crate::position::Position2d;
use crate::tile::Tile;

pub const BOARD_SIZE: usize = 25;

#[derive(Debug, Clone, PartialEq, Eq)]
pub struct Board {
    tiles: [Option<Tile>; BOARD_SIZE * BOARD_SIZE],
}

impl Board {
    pub fn get(&self, x: usize, y: usize) -> Option<Tile> {
        self.tiles[y * BOARD_SIZE + x]
    }

    pub fn set(&mut self, x: usize, y: usize, tile: Tile) {
        self.tiles[y * BOARD_SIZE + x] = Some(tile);
    }

    pub fn difference(&self, other: &Board) -> Vec<Position2d> {
        let mut diff = Vec::new();
        for x in 0..BOARD_SIZE {
            for y in 0..BOARD_SIZE {
                if self.get(x, y) != other.get(x, y) {
                    diff.push(Position2d::new(x, y));
                }
            }
        }
        diff
    }

    pub fn is_contiguous(positions: &[Position2d]) -> Option<bool> {
        let mut it = positions.iter();
        let first = *it.next()?;
        let mut second = *it.next()?;
        if first.x == second.x {
            // Vertical
            for &pos in it {
                if pos.x != first.x || pos.y != second.y + 1 {
                    return Some(false);
                }
                second = pos;
            }
            Some(true)
        } else if first.y == second.y {
            // Horizontal
            for &pos in it {
                if pos.y != first.y || pos.y != second.y + 1 {
                    return Some(false);
                }
                second = pos;
            }
            Some(true)
        } else {
            Some(false)
        }
    }
}

impl Default for Board {
    fn default() -> Self {
        Self {
            tiles: [None; BOARD_SIZE * BOARD_SIZE],
        }
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    fn positions(input: &[(usize, usize)]) -> Vec<Position2d> {
        input
            .iter()
            .map(|(x, y)| Position2d::new(*x, *y))
            .collect::<Vec<_>>()
    }

    #[test]
    fn test_is_contiguous() {
        assert_eq!(Board::is_contiguous(&[]), None);
        assert_eq!(Board::is_contiguous(&positions(&[(0, 0)])), None);
        assert_eq!(
            Board::is_contiguous(&positions(&[(0, 0), (0, 1), (0, 2)])),
            Some(true)
        );
        assert_eq!(
            Board::is_contiguous(&positions(&[(0, 0), (0, 1), (1, 3)])),
            Some(false)
        );
        assert_eq!(
            Board::is_contiguous(&positions(&[(0, 0), (0, 1), (0, 2), (1, 2)])),
            Some(false)
        );
    }
}
